@@ -4,13 +4,6 @@ import argparse
 import os
 from typing import TYPE_CHECKING
 
-from rich.progress import (
-    BarColumn,
-    DownloadColumn,
-    TimeRemainingColumn,
-    TransferSpeedColumn,
-)
-
 from pdm.cli.commands import build
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.commands.publish.package import PackageFile
@@ -21,7 +14,7 @@ from pdm.exceptions import PdmUsageError, PublishError
 from pdm.termui import logger
 
 if TYPE_CHECKING:
-    from requests import Response
+    from httpx import Response
 
     from pdm.project import Project
 
@@ -69,6 +62,17 @@ class Command(BaseCommand):
             dest="build",
             help="Don't build the package before publishing",
         )
+        parser.add_argument(
+            "-d",
+            "--dest",
+            help="The directory to upload the package from",
+            default="dist",
+        )
+        parser.add_argument(
+            "--skip-existing",
+            action="store_true",
+            help="Skip uploading files that already exist. This may not work with some repository implementations.",
+        )
         group = parser.add_mutually_exclusive_group()
         group.add_argument(
             "--no-very-ssl", action="store_false", dest="verify_ssl", help="Disable SSL verification", default=None
@@ -89,23 +93,43 @@ class Command(BaseCommand):
             p.sign(options.identity)
         return p
 
+    @staticmethod
+    def _skip_upload(response: Response) -> bool:
+        status = response.status_code
+        reason = response.reason_phrase.lower()
+        text = response.text.lower()
+
+        # Borrowed from https://github.com/pypa/twine/blob/main/twine/commands/upload.py#L149
+        return (
+            # pypiserver (https://pypi.org/project/pypiserver)
+            status == 409
+            # PyPI / TestPyPI / GCP Artifact Registry
+            or (status == 400 and any("already exist" in x for x in [reason, text]))
+            # Nexus Repository OSS (https://www.sonatype.com/nexus-repository-oss)
+            or (status == 400 and any("updating asset" in x for x in [reason, text]))
+            # Artifactory (https://jfrog.com/artifactory/)
+            or (status == 403 and "overwrite artifact" in text)
+            # Gitlab Enterprise Edition (https://about.gitlab.com)
+            or (status == 400 and "already been taken" in text)
+        )
+
     @staticmethod
     def _check_response(response: Response) -> None:
-        import requests
+        import httpx
 
         message = ""
-        if response.status_code == 410 and "pypi.python.org" in response.url:
+        if response.status_code == 410 and "pypi.python.org" in str(response.url):
             message = (
                 "Uploading to these sites is deprecated. "
                 "Try using https://upload.pypi.org/legacy/ "
                 "(or https://test.pypi.org/legacy/) instead."
             )
-        elif response.status_code == 405 and "pypi.org" in response.url:
+        elif response.status_code == 405 and "pypi.org" in str(response.url):
             message = "It appears you're trying to upload to pypi.org but have an invalid URL."
         else:
             try:
                 response.raise_for_status()
-            except requests.HTTPError as err:
+            except httpx.HTTPStatusError as err:
                 message = str(err)
                 if response.text:
                     logger.debug(response.text)
@@ -131,7 +155,7 @@ class Command(BaseCommand):
             config.ca_certs = ca_certs
         if options.verify_ssl is False:
             config.verify_ssl = options.verify_ssl
-        return Repository(project, config.url, config.username, config.password, config.ca_certs, config.verify_ssl)
+        return Repository(project, config)
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         hooks = HookManager(project, options.skip)
@@ -139,33 +163,27 @@ class Command(BaseCommand):
         hooks.try_emit("pre_publish")
 
         if options.build:
-            build.Command.do_build(project, hooks=hooks)
+            build.Command.do_build(project, dest=options.dest, hooks=hooks)
 
-        package_files = [str(p) for p in project.root.joinpath("dist").iterdir() if not p.name.endswith(".asc")]
-        signatures = {p.stem: str(p) for p in project.root.joinpath("dist").iterdir() if p.name.endswith(".asc")}
+        upload_dir = project.root.joinpath(options.dest)
+        package_files = [str(p) for p in upload_dir.iterdir() if not p.name.endswith(".asc")]
+        signatures = {p.stem: str(p) for p in upload_dir.iterdir() if p.name.endswith(".asc")}
 
         repository = self.get_repository(project, options)
         uploaded: list[PackageFile] = []
-        with project.core.ui.make_progress(
-            " [progress.percentage]{task.percentage:>3.0f}%",
-            BarColumn(),
-            DownloadColumn(),
-            "•",
-            TimeRemainingColumn(
-                compact=True,
-                elapsed_when_finished=True,
-            ),
-            "•",
-            TransferSpeedColumn(),
-        ) as progress, project.core.ui.logging("publish"):
+        with project.core.ui.logging("publish"):
             packages = sorted(
                 (self._make_package(p, signatures, options) for p in package_files),
                 # Upload wheels first if they exist.
                 key=lambda p: not p.base_filename.endswith(".whl"),
             )
             for package in packages:
-                resp = repository.upload(package, progress)
-                logger.debug("Response from %s:\n%s %s", resp.url, resp.status_code, resp.reason)
+                resp = repository.upload(package)
+                logger.debug("Response from %s:\n%s %s", resp.url, resp.status_code, resp.reason_phrase)
+
+                if options.skip_existing and self._skip_upload(resp):
+                    project.core.ui.warn(f"Skipping {package.base_filename} because it appears to already exist")
+                    continue
                 self._check_response(resp)
                 uploaded.append(package)
 
